#!/usr/bin/python
###################
# analyses nepenthes collected binaries
#
# requires scipy 0.5.2 and matplotlib 0.87.7
#
# NB: dir scan is much faster than per file scan
#
# Copyright Jean-Michel Philippe (c) 2006
#   <http://philipjm.free.fr/blog/>
#
# This is open source software released under the GPL license.
# The full text of this license is found in the file 'LICENSE',
# included with this source code package.
###################

###################
# importations
import sys, os, datetime, commands
from optparse import OptionParser
from pylab import *
import scipy.interpolate

###################
# constants
__version__ = '0.0.1'

###################
# get command line options
OptParser = OptionParser(usage="\npython %prog [options] directory [directory...]\nor\n./%prog [options] directory [directory...]")
OptParser.add_option(
	"-f", "--filter-coeff",
	type="float", dest="filtcoeff", default=10000.0,
	help="spline filter coefficient (default 10000.0)"
)
OptParser.add_option(
	"-v", "--verbose",
	action="store_true", dest="verbose", default=False,
	help="activates verbose mode"
)
OptParser.add_option(
	"-V", "--version",
	action="store_true", dest="version", default=False,
	help="just prints the version number"
)

# get parameters and args
(OptList, Args) = OptParser.parse_args()

# print version and quit
if OptList.version:
	print __version__
	sys.exit()

######################
# get clamav diagnosis and file date stamps
os.stat_float_times(True)
FileList = []
FileDates = []
ClamResults = {}
for DirName in Args:
	#TODO: test directory existence
	for Line in commands.getoutput("clamscan --no-summary '%s'" % DirName).split('\n'):
		ClamResult = Line.split(':')
		if len(ClamResult) != 2:
			continue
		FileName = ClamResult[0]
		ClamResults[FileName] = ClamResult[1].strip().replace(' FOUND','')
		
		FileList.append(FileName)
		FileCTime = os.stat(FileName).st_ctime
		FileDates.append( datetime.date.fromtimestamp(FileCTime) )

# in case no file is found
if not len(FileList):
	print "!!! no file found: nothing to do."
	sys.exit()

# perform simple stats
FilesPerDay = {}
ScanStatus = {}
k = 0
for FileName, FileDate in zip(FileList, FileDates):
	if FileDate in FilesPerDay:
		FilesPerDay[FileDate] += 1
	else:
		FilesPerDay[FileDate] = 1
	
	if ClamResults[FileName] == 'OK':
		if FileDate not in ScanStatus:
			ScanStatus[FileDate] = 0
	else:
		if FileDate in ScanStatus:
			ScanStatus[FileDate] += 1
		else:
			ScanStatus[FileDate] = 1

# print these stats
SortedDays = FilesPerDay.keys()
SortedDays.sort()
Data = []
for FileDate in SortedDays:
	Ratio = 100.0*ScanStatus[FileDate]/FilesPerDay[FileDate]
	Data.append( Ratio )
	if OptList.verbose:
		print "%s -> %d%% (%d/%d) detected as infected" % (FileDate.strftime('%d-%b-%y'), Ratio, ScanStatus[FileDate], FilesPerDay[FileDate])

# compute some extra data
xDates = asarray(map(date2num, SortedDays))
NbrPerDay = [FilesPerDay[x] for x in SortedDays]
spNbr = scipy.interpolate.UnivariateSpline(xDates, NbrPerDay)
spNbr.set_smoothing_factor(10000.0 * len(NbrPerDay))
spRatio = scipy.interpolate.UnivariateSpline(xDates, Data)
spRatio.set_smoothing_factor(10000.0 * len(Data))

# display bar charts
axes([0.1, 0.63, 0.85, 0.3])
bar(xDates, NbrPerDay)
plot(xDates, spNbr(xDates), 'r--', linewidth=1.5)
ax = gca()
setp(ax.get_xticklabels(), 'rotation', 30, 'horizontalalignment', 'right')
ax.xaxis.set_major_formatter(DateFormatter('%a %d/%m'))
title('Nepenthes new malware per day')
xlabel('Sampling date')
ylabel('Number of new malware')
legend(['spline filter', 'raw data'])
grid(1)

axes([0.1, 0.15, 0.85, 0.3])
bar(xDates, Data)
plot(xDates, spRatio(xDates), 'r--', linewidth=1.5)
ax = gca()
setp(ax.get_xticklabels(), 'rotation', 30, 'horizontalalignment', 'right')
ax.xaxis.set_major_formatter(DateFormatter('%a %d/%m'))
title('Nepenthes malware detected ratio per day')
xlabel('Sampling date')
ylabel('Malware ratio (%)')
legend(['spline filter', 'raw data'])
grid(1)

show()
